@@ -37,6 +37,7 @@ __copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
+import os
 import json
 import base64
 import string
@@ -50,6 +51,7 @@ from . import legacy
 from . import typesf
 from . import config
 from . import exceptions
+from . import structures
 
 TIMEOUT = 60
 """ The timeout in seconds to be used for the blocking
@@ -71,12 +73,18 @@ new authentication try will be performed """
 
 ACCESS_LOCK = threading.RLock()
 """ Global access lock used for locking global operations
-that required thread safety under the HTTP infra-structure """
+that require thread safety under the HTTP infra-structure """
 
-def try_auth(auth_callback, params, headers = None):
-    if not auth_callback: raise
-    if headers == None: headers = dict()
-    auth_callback(params, headers)
+def file_g(path, chunk = 40960):
+    yield os.path.getsize(path)
+    file = open(path, "rb")
+    try:
+        while True:
+            data = file.read(chunk)
+            if not data: break
+            yield data
+    finally:
+        file.close()
 
 def get_f(*args, **kwargs):
     name = kwargs.pop("name", "default")
@@ -232,6 +240,11 @@ def patch(
         **kwargs
     )
 
+def _try_auth(auth_callback, params, headers = None):
+    if not auth_callback: raise
+    if headers == None: headers = dict()
+    auth_callback(params, headers)
+
 def _method(method, *args, **kwargs):
     try:
         auth_callback = kwargs.pop("auth_callback", None)
@@ -241,7 +254,7 @@ def _method(method, *args, **kwargs):
             params = kwargs.get("params", None)
             headers = kwargs.get("headers", None)
             if not error.code in AUTH_ERRORS : raise
-            try_auth(auth_callback, params, headers)
+            _try_auth(auth_callback, params, headers)
             result = method(*args, **kwargs)
         except legacy.HTTPError as error:
             code = error.getcode()
@@ -414,8 +427,16 @@ def _method_empty(
     url = str(url)
 
     _method_callback(handle, kwargs)
+
+    # runs the concrete resolution method (taking into account the adapter)
+    # providing it with the required parameters for request execution
     file = _resolve(url, name, headers, None, silent, timeout, **kwargs)
+
+    # verifies if the resulting "file" from the resolution process is either
+    # an invalid or tuple value and if that's the case as the request has
+    # probably been deferred for asynchronous execution
     if file == None: return file
+    if isinstance(file, tuple): return file
 
     try: result = file.read()
     finally: file.close()
@@ -484,11 +505,14 @@ def _method_payload(
         data = data_e
         mime = mime or "application/x-www-form-urlencoded"
 
-    data = legacy.bytes(data)
-    length = len(data) if data else 0
+    if legacy.is_unicode(data): data = legacy.bytes(data, force = True)
+
+    if not data: length = 0
+    elif legacy.is_bytes(data): length = len(data)
+    else: length = -1
 
     headers = dict(headers) if headers else dict()
-    headers["Content-Length"] = str(length)
+    if not length == -1: headers["Content-Length"] = str(length)
     if mime: headers["Content-Type"] = mime
     if host: headers["Host"] = host
     if authorization: headers["Authorization"] = "Basic %s" % authorization
@@ -602,6 +626,10 @@ def _resolve(*args, **kwargs):
     return result
 
 def _resolve_legacy(url, method, headers, data, silent, timeout, **kwargs):
+    is_generator = not data == None and legacy.is_generator(data)
+    if is_generator: next(data); data = b"".join(data)
+    is_file = hasattr(data, "tell")
+    if is_file: data = data.read()
     opener = legacy.build_opener(legacy.HTTPHandler)
     request = legacy.Request(url, data = data, headers = headers)
     request.get_method = lambda: method
@@ -617,6 +645,14 @@ def _resolve_requests(url, method, headers, data, silent, timeout, **kwargs):
     reuse = kwargs.get("reuse", True)
     connections = kwargs.get("connections", 256)
 
+    # verifies if the provided data is a generator, assumes that if the
+    # data is not invalid and is of type generator then it's a generator
+    # and then if that's the case encapsulates this size based generator
+    # into a generator based file-like object so that it can be used inside
+    # the request infra-structure (as it accepts only file objects)
+    is_generator = not data == None and legacy.is_generator(data)
+    if is_generator: data = structures.GeneratorFile(data)
+
     # verifies if the session for the requests infra-structure is
     # already created and if that's not the case and the re-use
     # flag is sets creates a new session for the requested settings
@@ -690,6 +726,8 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
     asynchronous = kwargs.get("asynchronous", asynchronous)
     use_file = kwargs.get("use_file", False)
     callback = kwargs.get("callback", None)
+    callback_init = kwargs.get("callback_init", None)
+    callback_open = kwargs.get("callback_open", None)
     callback_headers = kwargs.get("callback_headers", None)
     callback_data = kwargs.get("callback_data", None)
     callback_result = kwargs.get("callback_result", None)
@@ -703,10 +741,14 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
     # HTTP client taking into account a possible async method request
     extra = _async_netius(
         callback = callback,
+        callback_init = callback_init,
+        callback_open = callback_open,
         callback_headers = callback_headers,
         callback_data = callback_data,
         callback_result = callback_result
     ) if asynchronous else dict(
+        on_init = lambda c: callback_init and callback_init(c),
+        on_open = lambda c: callback_open and callback_open(c),
         on_headers = lambda c, p: callback_headers and callback_headers(p.headers),
         on_data = lambda c, p, d: callback_data and callback_data(d),
         on_result = lambda c, p, r: callback_result and callback_result(r)
@@ -729,9 +771,9 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
         **extra
     )
 
-    # if the async mode is defined an invalid value is returned immediately
+    # if the async mode is defined the result (tuple) is returned immediately
     # as the processing will be taking place latter (on callback)
-    if asynchronous: return None
+    if asynchronous: return result
 
     # tries to retrieve any possible error coming from the result object
     # if this happens it means an exception has been raised internally and
@@ -769,7 +811,7 @@ def _client_netius(level = logging.CRITICAL):
     global _netius_clients
 
     # retrieves the reference to the current thread and uses the value
-    # to retrieve the thread identifier (tid) for it, to be used in the
+    # to retrieve the thread identifier (TID) for it, to be used in the
     # identification of the client resource associated with it
     tid = threading.current_thread().ident
 
@@ -784,21 +826,13 @@ def _client_netius(level = logging.CRITICAL):
     finally:
         ACCESS_LOCK.release()
 
-    # in case the retrieved client is not valid (stopped) then
-    # it must be invalidated as it cannot be re-used
-    if netius_client and netius_client.is_stopped(): netius_client = None
-
     # in case a previously created netius client has been retrieved
     # returns it to the caller method for proper re-usage
     if netius_client: return netius_client
 
     # creates the "new" HTTP client for the current thread and registers
     # it under the netius client structure so that it may be re-used
-    netius_client = netius.clients.HTTPClient(
-        thread = False,
-        auto_pause = True,
-        level = level
-    )
+    netius_client = netius.clients.HTTPClient(auto_release = False)
     _netius_clients[tid] = netius_client
 
     # in case this is the first registration of the dictionary a new on
@@ -809,6 +843,8 @@ def _client_netius(level = logging.CRITICAL):
 
 def _async_netius(
     callback = None,
+    callback_init = None,
+    callback_open = None,
     callback_headers = None,
     callback_data = None,
     callback_result = None
@@ -818,28 +854,36 @@ def _async_netius(
     buffer = []
     extra = dict()
 
-    def _on_close(connection):
+    def _on_init(protocol):
+        callback_init and callback_init(protocol)
+
+    def _on_open(protocol):
+        callback_open and callback_open(protocol)
+
+    def _on_close(protocol):
         callback and callback(None)
 
-    def _on_headers(client, parser):
+    def _on_headers(protocol, parser):
         callback_headers and callback_headers(parser.headers)
 
-    def _on_data(client, parser, data):
+    def _on_data(protocol, parser, data):
         data = data
         data and buffer.append(data)
         callback_data and callback_data(data)
 
-    def _on_result(client, parser, result):
+    def _on_result(protocol, parser, result):
         callback_result and callback_result(result)
 
-    def _callback(connection, parser, message):
-        result = netius.clients.HTTPClient.set_request(parser, buffer)
+    def _callback(protocol, parser, message):
+        result = netius.clients.HTTPProtocol.set_request(parser, buffer)
         response = netius.clients.HTTPClient.to_response(result)
         callback and callback(response)
 
     extra["callback"] = _callback
-    extra["on_close"] = _on_close
     extra["on_data"] = _on_data
+    if callback_init: extra["_on_init"] = _on_init
+    if callback_open: extra["on_open"] = _on_open
+    if callback: extra["on_close"] = _on_close
     if callback_headers: extra["on_headers"] = _on_headers
     if callback_result: extra["on_result"] = _on_result
 
@@ -847,7 +891,8 @@ def _async_netius(
 
 def _cleanup_netius():
     global _netius_clients
-    for netius_client in _netius_clients.values(): netius_client.cleanup()
+    for netius_client in _netius_clients.values():
+        netius_client.cleanup()
     del _netius_clients
 
 def _parse_url(url):
