@@ -82,7 +82,7 @@ def dumps(*args):
 def _redis(verify = True):
     if verify: util.verify(
         not redis == None,
-        message = "redis library not available",
+        message = "RedisPy library not available",
         exception = exceptions.OperationalError
     )
     return redis
